@@ -34,18 +34,21 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_PTHREAD
+#include <pthread.h>
+#endif
+
 #define DEFAULT_PAGE                  "index.html"
 #define DEFAULT_RECV_SIZE             1024
 #define DEFAULT_READ_SIZE             8192
 #define MAX_HEADER_LEN                4096
 #define MAX_KEEP_ALIVE                100
-#define MAX_NEW_CONNECTIONS_PER_STEP  16
+#define MAX_NEW_CONNECTIONS_PER_STEP  100
 
+#define EXIT_CANT_CREATE_SERVER_SOCKET4 10
 
 #define CRLF "\r\n"
 
-#define NEW(type_) (type_ *) malloc (sizeof(type_))
-
 
 /* Global mini-buffer:
  * for quick usage
@@ -61,10 +64,10 @@ typedef enum {
 	ret_eof,
 	ret_eof_have_data,
 	ret_file_not_found,
-	ret_eagain
+	ret_eagain,
+	ret_ok_and_sent
 } ret_t;
-
-#if 1
+	
 #define return_if_fail(expr,ret) \
 	if (!(expr)) {                                                    \
 		fprintf (stderr,                                          \
@@ -75,9 +78,7 @@ typedef enum {
                           #expr);                                         \
 	        return (ret);                                             \
 	};
-#else
-#define return_if_fail(expr,ret)
-#endif
+
 
 #define SHOULDNT_HAPPEN \
 	do { fprintf (stderr, "file %s: line %d (%s): this shouldn't happend\n", \
@@ -93,6 +94,18 @@ typedef enum {
 	cherokee_ ## type ## _new (& obj );      \
 	return_if_fail (obj != NULL, ret_nomem);
 
+#define CHEROKEE_TEMP(obj, size)                 \
+        const unsigned int obj ## _size = size;  \
+	char obj[size];
+
+
+#ifdef HAVE_PTHREADS
+#define CHEROKEE_MUTEX_LOCK(m)   pthread_mutex_lock(m)
+#define CHEROKEE_MUTEX_UNLOCK(m) pthread_mutex_unlock(m)
+#else
+#define CHEROKEE_MUTEX_LOCK(m)
+#define CHEROKEE_MUTEX_UNLOCK(m)
+#endif
 
 void PRINT_ERROR (const char *format, ...);
 
