@@ -27,15 +27,17 @@
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/types.h>
 #include <pwd.h>
 #include <time.h>
 
 /* inet_ntoa()
  */
+#include <sys/types.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
+
 #include <arpa/inet.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 
 #include "log.h"
 #include "handler.h"
@@ -46,16 +48,6 @@
 #include "plugin_table_entry.h"
 #include "encoder_table.h"
 
-
-static char *month[]   = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-			  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", NULL};
-
-static char *method[]  = {"GET", "POST", "HEAD", "UNKNOWN", NULL};
-
-static char *version[] = {"HTTP/0.9", "HTTP/1.0", "HTTP/1.1", "UNKNOWN", NULL};
-
-
-
 ret_t
 cherokee_connection_new  (cherokee_connection_t **cnt)
 {
@@ -64,26 +56,27 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	cherokee_buffer_new (&n->buffer);
 	   
 	INIT_LIST_HEAD((list_t*)n);
-	n->socket          = -1;
+	n->tcp_cork        = 0;
 	n->error_code      = http_ok;
-	n->status          = stat_reading;
 	n->phase           = phase_reading_header;
 	n->method          = http_unknown;
 	n->version         = http_version_unknown;
 	n->handler         = NULL; 
 	n->encoder         = NULL;
+	n->logger_ref      = NULL;
 	n->keep_alive      = 0;
-	n->timeout         = time(NULL) + 15;
+	n->timeout         = time(NULL) + 15;   // CHECK ME !!
 	n->range_start     = 0;
 	n->range_end       = 0;
-
 	
 	cherokee_buffer_new (&n->local_directory);
 	cherokee_buffer_new (&n->web_directory);
 	cherokee_buffer_new (&n->request);
 	cherokee_buffer_new (&n->redirect);
 	cherokee_buffer_new (&n->host);
-	
+
+	cherokee_socket_new (&n->socket);
+
 	*cnt = n;
 	return ret_ok;
 }
@@ -92,6 +85,7 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 ret_t
 cherokee_connection_free (cherokee_connection_t  *cnt)
 {
+	   cherokee_socket_free (cnt->socket);
 	   cherokee_buffer_free (cnt->buffer);
 
 	   if (cnt->handler != NULL) {
@@ -110,6 +104,10 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 	   cherokee_buffer_free (cnt->redirect);
 	   cherokee_buffer_free (cnt->host);
 
+	   if(cnt->query_string) {
+		   cherokee_buffer_free(cnt->query_string);
+	   }
+
 	   free (cnt);
 	   return ret_ok;
 }
@@ -118,7 +116,6 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 ret_t
 cherokee_connection_clean (cherokee_connection_t *cnt)
 {	   
-	   cnt->status      = stat_reading;
 	   cnt->phase       = phase_reading_header;
 	   cnt->version     = http_version_unknown;
 	   cnt->method      = http_unknown;
@@ -126,6 +123,8 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 	   cnt->timeout     = -1;
 	   cnt->range_start = 0;
 	   cnt->range_end   = 0;
+	   cnt->logger_ref  = NULL;
+	   cnt->tcp_cork    = 0;
 
 	   if (cnt->handler != NULL) {
 		   cherokee_handler_free (cnt->handler);
@@ -151,10 +150,14 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 ret_t 
 cherokee_connection_mrproper (cherokee_connection_t *cnt)
 {
-	cnt->socket     = -1;
+	ret_t ret;
+
 	cnt->keep_alive = 0;
 
-	return cherokee_connection_clean (cnt);
+	ret  = cherokee_connection_clean (cnt);
+	ret |= cherokee_socket_close (cnt->socket);
+
+	return ret;
 }
 
 
@@ -197,13 +200,12 @@ cherokee_connection_send_header (cherokee_connection_t  *cnt, int hideversion)
 	case http_version_09:
 		cherokee_buffer_add (cnt->buffer, "HTTP/0.9 ", 9); 
 		break;
-	case http_version_10:
-		cherokee_buffer_add (cnt->buffer, "HTTP/1.0 ", 9); 
-		break;
 	case http_version_11:
 		cherokee_buffer_add (cnt->buffer, "HTTP/1.1 ", 9); 
 		break;
+	case http_version_10:
 	default:
+		cherokee_buffer_add (cnt->buffer, "HTTP/1.0 ", 9); 
 		break;
 	}
 	
@@ -255,13 +257,21 @@ cherokee_connection_send_header (cherokee_connection_t  *cnt, int hideversion)
 		cherokee_buffer_add (cnt->buffer, "Connection: close"CRLF, 19);
 	}
 	
+	/* If it's an error page we've to add the content-type header
+	 */
+	if (! http_type_200(cnt->error_code)) {
+		cherokee_buffer_add (cnt->buffer, "Content-Type: text/html"CRLF, 25);		
+	}
+
 	/* Add the response header ends
 	 */
 	cherokee_buffer_add (cnt->buffer, CRLF, 2);
 	
 	/* Send it
 	 */
-	ret = write (cnt->socket, cnt->buffer->buf, cnt->buffer->len);
+	ret = write (SOCKET_FD(cnt->socket), cnt->buffer->buf,
+		     cnt->buffer->len);
+
 	cherokee_buffer_make_empty  (cnt->buffer);
 	
 	if (ret > 0)
@@ -277,7 +287,7 @@ cherokee_connection_recv (cherokee_connection_t  *cnt)
 	int  readed;
 	char tmp[DEFAULT_RECV_SIZE];
 	
-	readed = read (cnt->socket, tmp, DEFAULT_RECV_SIZE);
+	readed = read (SOCKET_FD(cnt->socket), tmp, DEFAULT_RECV_SIZE);
 
 	if (readed > 0) {
 		cherokee_buffer_add (cnt->buffer, tmp, readed);
@@ -320,10 +330,29 @@ cherokee_connection_reading_check (cherokee_connection_t *cnt)
 }
 
 
-ret_t
-cherokee_connection_set_socket (cherokee_connection_t *cnt, int socket)
+ret_t 
+cherokee_connection_set_cork (cherokee_connection_t *cnt, int enable)
 {
-	cnt->socket = socket;
+	int on = 0;
+	int fd;
+
+#ifdef HAVE_TCP_CORK
+	fd = SOCKET_FD(cnt->socket);
+	if (enable) {
+		setsockopt(fd, IPPROTO_TCP, TCP_NODELAY,  &on, sizeof on);
+
+		on = 1;
+		setsockopt(fd, IPPROTO_TCP, TCP_CORK,  &on, sizeof on);
+	} else {
+		setsockopt(fd, IPPROTO_TCP, TCP_CORK,  &on, sizeof on);
+
+		on = 1;
+		setsockopt(fd, IPPROTO_TCP, TCP_NODELAY,  &on, sizeof on);
+	}
+
+	cnt->tcp_cork = enable;
+#endif
+
 	return ret_ok;
 }
 
@@ -332,20 +361,31 @@ ret_t
 cherokee_connection_send (cherokee_connection_t *cnt)
 {
 	int sent;
-	
-	sent = write (cnt->socket, cnt->buffer->buf, cnt->buffer->len);
-	
+
+	sent = write (SOCKET_FD(cnt->socket), cnt->buffer->buf, cnt->buffer->len);
+
 	if (sent > 0) {
 		if (sent == cnt->buffer->len) {
 			cherokee_buffer_make_empty (cnt->buffer);
 		} else {
 			cherokee_buffer_move_to_begin (cnt->buffer, sent);
 		}
-		
-	} else if (sent == 0) {
+
+		/* If this connection has a handler without content-length support
+		 * it has to count the bytes sent
+		 */
+		if (HANDLER_SUPPORT_LENGTH(cnt->handler)) {
+			cnt->range_end += sent;
+		}
+
+		return ret_ok;		
+	}
+	
+	if (sent == 0) {
 		return ret_eof;
-		
-	} else {
+	}
+
+	if (sent < 0) {
 		switch (errno) {
 		case EAGAIN:
 			return ret_eagain;
@@ -355,18 +395,13 @@ cherokee_connection_send (cherokee_connection_t *cnt)
 			return ret_error;
 		}
 	}
-	
-	return ret_ok;
 }
 
 
 ret_t
 cherokee_connection_close (cherokee_connection_t *cnt)
 {
-	if (close (cnt->socket) == 0)
-		return ret_ok;
-	else
-		return ret_error;
+	return cherokee_socket_close (cnt->socket);
 }
 
 
@@ -418,16 +453,23 @@ get_host (cherokee_connection_t *cnt,
 {
 	/* ptr - Header at the "Host:" position 
 	 */
-
-	char *end;
+	char *end, *end2;
 	int   size;
 
 	end = strchr (ptr, '\r');
 	if (end == NULL) {
 		return ret_error;
-	}	
-	size = (end - ptr);
+	}
 
+	/* Drop the port if present
+	 * Eg: www.alobbs.com:8080 -> www.alobbs.com
+	 */ 
+	end2 = strchr (ptr, ':');
+	if (end2 && (end2 < end))  {
+		end = end2;
+	}
+
+	size = (end - ptr);
 	return cherokee_buffer_add (cnt->host, ptr, size);
 }
 
@@ -543,11 +585,22 @@ get_uri (cherokee_connection_t *cnt)
 	 */
 	ptr = index (cnt->buffer->buf + begin, '?');
 	if (ptr != NULL) {
-		end = ptr - cnt->buffer->buf;
-		
-		// TODO: Parse the parameters
+		int qs_len;
+
+		/* Save the query string  */
+		++ptr;
+		qs_len = strcspn(ptr, "\n\r ");
+
+		cherokee_buffer_new(&cnt->query_string);
+		cherokee_buffer_add(cnt->query_string, ptr, qs_len);
+
+		/* Sets the end to the filename */
+		end = ptr - cnt->buffer->buf - 1;
+
+	} else {
+		cnt->query_string = NULL;
 	}
-	
+
 	/* Dup the request
 	 */
 	cherokee_buffer_add (cnt->request, cnt->buffer->buf + begin, end-begin);
@@ -558,15 +611,13 @@ get_uri (cherokee_connection_t *cnt)
 
 	/* Look for "Host:"
 	 */
-	if (cnt->version == http_version_11) {
-		if ((ptr = strstr (cnt->buffer->buf, "Host: "))) {
-			ptr += 6;
-			get_host (cnt, ptr);
-
-		} else {
-
-			/* HTTP/1.1 connections *must* have an `Host' header
-			 */
+	if ((ptr = strstr (cnt->buffer->buf, CRLF "Host: "))) {
+		ptr += 8;
+		get_host (cnt, ptr);
+	} else {
+		/* HTTP/1.1 connections *must* have an `Host' header
+		 */
+		if (cnt->version == http_version_11) {
 			cnt->error_code = http_bad_request;
 			return ret_error;
 		}
@@ -720,10 +771,33 @@ get_range (cherokee_connection_t *cnt,
 }
 
 
+static void
+delete_double_slash (cherokee_buffer_t *buffer)
+{
+	char *a      = buffer->buf;
+	int   offset = 0;
+
+	if (buffer->len < 2) return;
+
+	do {
+		if ((a[0] == '/') && (a[offset+1] == '/')) {
+			offset++;
+			continue;
+		}
+		
+		a[0] = a[offset];
+		a++;
+
+	} while ((a && *a != '\0') && (a < buffer->buf + buffer->len));
+
+	buffer->len -= offset;
+}
+
+
 ret_t 
 cherokee_connection_get_request (cherokee_connection_t *cnt)
 {
-	ret_t ret;
+	ret_t  ret;
 
         /* Get the request     
 	 */
@@ -747,6 +821,10 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 		return ret_error;		
 	}
 
+	/* Look for "//" 
+	 */
+	delete_double_slash (cnt->request);
+
 	cnt->error_code = http_ok;
 	return ret_ok;	
 }
@@ -904,7 +982,7 @@ cherokee_connection_send_response_page (cherokee_connection_t *cnt, int hidevers
    
 	   /* Send it
 	    */
-	   ret = write (cnt->socket, cnt->buffer->buf, cnt->buffer->len);
+	   ret = write (SOCKET_FD(cnt->socket), cnt->buffer->buf, cnt->buffer->len);
 	   cherokee_buffer_make_empty  (cnt->buffer);
 
 	   if (ret > 0)
@@ -914,40 +992,33 @@ cherokee_connection_send_response_page (cherokee_connection_t *cnt, int hidevers
 }
 
 
-
-void
-cherokee_connection_log (cherokee_connection_t *cnt, time_t bogo_now)
+ret_t 
+cherokee_connection_log_or_delay (cherokee_connection_t *conn)
 {
-        struct tm *timep;
-	long int   z;
-	
-        timep = (struct tm *) localtime(&bogo_now);
+	ret_t ret = ret_ok;
 
-#ifdef HAVE_INT_TIMEZONE
-	z = - (timezone / 60);
-#else
-#warning TODO
-	z = 0;
-#endif
+	conn->log_at_end = ! HANDLER_SUPPORT_LENGTH(conn->handler);
 
-	snprintf (gbl_buffer, gbl_buffer_size, "%s - - [%02d/%s/%d:%02d:%02d:%02d %c%02d%02d] \"%s %s %s\" %d %d",
+	if (conn->log_at_end == 0) {
+		if (conn->logger_ref != NULL) {
+			ret = cherokee_logger_write_access (conn->logger_ref, conn);
+		}
+	}
 
-		  inet_ntoa(cnt->addr_in.sin_addr),
+	return ret;
+}
 
-		  timep->tm_mday, 
-		  month[timep->tm_mon], 
-		  1900 + timep->tm_year,
-		  timep->tm_hour, 
-		  timep->tm_min, 
-		  timep->tm_sec,
-		  (z < 0) ? '-' : '+', 
-		  (int) z/60, 
-		  (int) z%60, 
-		  method[cnt->method],
-		  cnt->request->buf, 
-		  version[cnt->version], 
-		  cnt->error_code,
-		  cnt->range_end - cnt->range_start);
 
-	cherokee_log (LOG_INFO, gbl_buffer);
+ret_t 
+cherokee_connection_log_delayed (cherokee_connection_t *conn)
+{
+	ret_t ret = ret_ok;
+
+	if (conn->log_at_end) {
+		if (conn->logger_ref != NULL) {
+			ret = cherokee_logger_write_access (conn->logger_ref, conn);
+		}		
+	}
+
+	return ret_ok;
 }
