@@ -35,25 +35,20 @@
 #include <sys/types.h>	
 #include <sys/socket.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
 
 #include "list.h"
 #include "http.h"
 #include "common.h"
 #include "buffer.h"
 #include "handler.h"
-#include "handler_file.h"
 #include "plugin_table.h"
 #include "virtual_server.h"
 #include "encoder_table.h"
 #include "plugin_table_entry.h"
+#include "socket.h"
 
 
-typedef enum {
-	stat_reading = 0,
-	stat_writing = 1,
-	stat_closed
-} cherokee_connection_status_t;
-
 typedef enum {
 	phase_reading_header,
 	phase_processing_header,
@@ -62,23 +57,22 @@ typedef enum {
 	phase_steping,
 } cherokee_connection_phase_t;
 
-
 typedef struct {
 	struct list_head              list;
 	
 	void                         *server;
 
-	int                           socket;
-	struct sockaddr_in            addr_in;
+	cherokee_socket_t            *socket;
+	int                           tcp_cork;
 	
-	cherokee_connection_status_t  status;
 	cherokee_connection_phase_t   phase;
 
-
 	cherokee_http_version_t       version;
 	cherokee_http_method_t        method;
 
 	cherokee_encoder_t           *encoder;
+	cherokee_logger_t            *logger_ref;
+	int                           log_at_end;
 
 	cherokee_http_t               error_code;
 	
@@ -93,6 +87,7 @@ typedef struct {
 	cherokee_buffer_t            *local_directory;  /* /var/www/  or  /home/alo/public_html/ */
 	cherokee_buffer_t            *web_directory;    /* /cherokee/ */
 	cherokee_buffer_t            *request;          /* /dir/file */
+	cherokee_buffer_t            *query_string;
 	cherokee_buffer_t            *host;
 
 	time_t                        timeout;
@@ -103,22 +98,25 @@ typedef struct {
 } cherokee_connection_t;
 
 
-#define CONN(c)     ((cherokee_connection_t *)(c))
-#define CONN_SRV(c) ((cherokee_server_t *) CONN(c)->server)
+#define CONN(c)      ((cherokee_connection_t *)(c))
+#define CONN_SRV(c)  ((cherokee_server_t *) CONN(c)->server)
+#define CONN_SOCK(c) ((cherokee_socket_t *) CONN(c)->socket)
 
 
 ret_t cherokee_connection_new        (cherokee_connection_t **cnt);
 ret_t cherokee_connection_free       (cherokee_connection_t  *cnt);
 ret_t cherokee_connection_clean      (cherokee_connection_t  *cnt);
 ret_t cherokee_connection_mrproper   (cherokee_connection_t  *cnt);
-ret_t cherokee_connection_set_socket (cherokee_connection_t  *cnt, int socket);
-void  cherokee_connection_log        (cherokee_connection_t  *cnt, time_t bogo_now);
+
+/*	Socket
+ */
+ret_t cherokee_connection_set_cork   (cherokee_connection_t *cnt, int enable);
 
 /* 	Send & Recv
  */
-ret_t cherokee_connection_recv  (cherokee_connection_t  *cnt);
-ret_t cherokee_connection_send  (cherokee_connection_t  *cnt);
-ret_t cherokee_connection_close (cherokee_connection_t  *cnt);
+ret_t cherokee_connection_recv  (cherokee_connection_t *cnt);
+ret_t cherokee_connection_send  (cherokee_connection_t *cnt);
+ret_t cherokee_connection_close (cherokee_connection_t *cnt);
 
 /* 	Sending
  */
@@ -127,9 +125,9 @@ ret_t cherokee_connection_send_response_page (cherokee_connection_t *cnt, int hi
 
 /*	Iteration
  */
-ret_t cherokee_connection_open_request  (cherokee_connection_t  *cnt);
-ret_t cherokee_connection_reading_check (cherokee_connection_t  *cnt);
-ret_t cherokee_connection_step          (cherokee_connection_t  *cnt);
+ret_t cherokee_connection_open_request  (cherokee_connection_t *cnt);
+ret_t cherokee_connection_reading_check (cherokee_connection_t *cnt);
+ret_t cherokee_connection_step          (cherokee_connection_t *cnt);
 
 ret_t cherokee_connection_get_plugin_entry (cherokee_connection_t  *cnt, cherokee_plugin_table_t *plugins, cherokee_plugin_table_entry_t **plugin_entry);
 ret_t cherokee_connection_create_handler   (cherokee_connection_t  *cnt, cherokee_plugin_table_entry_t *plugin_entry);
@@ -140,9 +138,17 @@ int   cherokee_connection_eoh          (cherokee_connection_t  *cnt);
 ret_t cherokee_connection_get_request  (cherokee_connection_t  *cnt);
 ret_t cherokee_connection_parse_header (cherokee_connection_t  *cnt, cherokee_encoder_table_t *encoders);
 
+
+/* 	Log
+ */
+ret_t cherokee_connection_log_or_delay (cherokee_connection_t  *cnt);
+ret_t cherokee_connection_log_delayed  (cherokee_connection_t  *cnt);
+
+
 int   cherokee_connection_is_userdir                    (cherokee_connection_t  *cnt);
 ret_t cherokee_connection_build_local_directory         (cherokee_connection_t  *cnt, cherokee_virtual_server_t *vsrv, cherokee_plugin_table_entry_t *entry);
 ret_t cherokee_connection_build_local_directory_userdir (cherokee_connection_t  *cnt, char *userdir);
 
 
+
 #endif /* __CHEROKEE_CONNECTION_H__  */
