@@ -32,7 +32,7 @@ ret_t
 cherokee_encoder_table_new  (cherokee_encoder_table_t **et)
 {
 	   int i;
-	   cherokee_encoder_table_t *n = (cherokee_encoder_table_t *) malloc (sizeof(cherokee_encoder_table_t));
+	   CHEROKEE_NEW_STRUCT(n, encoder_table);
 
 	   /* Init the table
 	    */
@@ -54,36 +54,12 @@ cherokee_encoder_table_new  (cherokee_encoder_table_t **et)
 	   return ret_ok;
 }
 
-/*
-static inline encoder_func_new_t
-load_encoder_library (char *plugin_name)
-{
-	char lib[200];
-	char sym[200];
-
-	void (*init) (void);
-	encoder_func_new_t ret;
-
- 	snprintf (lib, 200, "libcherokee_%s." SO_SUFFIX, plugin_name);
-	snprintf (sym, 200, "%s_init", plugin_name);
-
-	init = get_sym_from_dylib (lib, sym);
-	if (init != NULL) {
-		init();
-	}
-
- 	snprintf (sym, 200, "cherokee_encoder_%s_new", plugin_name);
-	ret = get_sym_from_dylib (lib, sym);
-	
-	return ret;
-}
-*/
 
 static inline encoder_func_new_t
 load_encoder_library (char *plugin_name)
 {
 	cherokee_library_t *clib;
-	encoder_func_new_t ret;
+	func_new_t ret;
 	char sym[200];
 	
 	cherokee_library_new (&clib, plugin_name);
@@ -93,7 +69,7 @@ load_encoder_library (char *plugin_name)
 
 	cherokee_library_free (clib);
 	
-	return ret;
+	return (encoder_func_new_t) ret;
 }
 
 
@@ -178,7 +154,7 @@ cherokee_encoder_table_new_encoder (cherokee_encoder_table_t *et, char *encoder,
 		ret = cherokee_encoder_entry_get_new_func (entry, &new_func);
 		if (ret != ret_ok) return ret;
 		
-		ret = new_func (new_encoder);
+		ret = new_func ((void **)new_encoder);
 		if (ret != ret_ok) return ret;
 	}
 
