@@ -23,8 +23,8 @@
  * USA
  */
 
-#ifndef __CHEROKEE2_FD_POLL_H__
-#define __CHEROKEE2_FD_POLL_H__
+#ifndef __CHEROKEE_FD_POLL_H__
+#define __CHEROKEE_FD_POLL_H__
 
 #ifdef USE_POOL_EMULATION
 # include "poll_emu.h"
@@ -36,6 +36,10 @@
 # endif
 #endif
 
+#ifdef HAVE_PTHREAD
+#include <pthread.h>
+#endif
+
 #include "common.h"
 
 
@@ -45,11 +49,15 @@ typedef enum {
 } cherokee_fdpoll_rw_t;
 
 typedef struct {
-	   int nfiles;
-	   
-	   struct pollfd* pollfds;
-	   int npollfds;
-	   int* fdidx;
+	int            nfiles;
+	struct pollfd *pollfds;
+	int            npollfds;
+	int           *fdidx;
+	
+#ifdef HAVE_PTHREAD
+	pthread_rwlock_t rwlock;
+#endif
+
 } cherokee_fdpoll_t;
 
 ret_t cherokee_fdpoll_new      (cherokee_fdpoll_t **fdp);
@@ -57,8 +65,8 @@ ret_t cherokee_fdpoll_free     (cherokee_fdpoll_t  *fdp);
 
 ret_t cherokee_fdpoll_add      (cherokee_fdpoll_t  *fdp, int fd, int rw);
 ret_t cherokee_fdpoll_del      (cherokee_fdpoll_t  *fdp, int fd);
-ret_t cherokee_fdpoll_set_mode (cherokee_fdpoll_t  *fdp, int fd, int rw);
+void  cherokee_fdpoll_set_mode (cherokee_fdpoll_t  *fdp, int fd, int rw);
 int   cherokee_fdpoll_check    (cherokee_fdpoll_t  *fdp, int fd, int rw);
 int   cherokee_fdpoll_watch    (cherokee_fdpoll_t  *fdp, int timeout_msecs);
 
-#endif /* __CHEROKEE2_FD_POLL_H__ */
+#endif /* __CHEROKEE_FD_POLL_H__ */
