@@ -70,9 +70,11 @@ typedef struct {
 	   
 } cherokee_handler_t;
 
-#define HANDLER(x)       ((cherokee_handler_t *)(x))
-#define HANDLER_CONN(h)  (CONN(HANDLER(h)->connection))
-#define HANDLER_SRV(h)   (CONN_SRV(HANDLER_CONN(h)))
+#define HANDLER(x)                ((cherokee_handler_t *)(x))
+#define HANDLER_CONN(h)           (CONN(HANDLER(h)->connection))
+#define HANDLER_SRV(h)            (CONN_SRV(HANDLER_CONN(h)))
+#define HANDLER_SUPPORT_RANGE(h)  (HANDLER(h)->support & hsupport_range)
+#define HANDLER_SUPPORT_LENGTH(h) (HANDLER(h)->support & hsupport_length)
 
 ret_t cherokee_handler_init_base   (cherokee_handler_t  *hdl, void *conn);
 
