@@ -24,12 +24,19 @@
  */
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 
+#include "common.h"
 #include "mime.h"
 #include "handler_file.h"
 #include "connection.h"
 
+#if HAVE_SENDFILE
+#include <sys/sendfile.h>
+#endif
+
 
 ret_t
 cherokee_handler_file_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties)
@@ -68,19 +75,25 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 
 	conn = CONN(HANDLER(n)->connection);
 
-	cherokee_buffer_add (conn->local_directory, conn->request->buf, conn->request->len);
+	cherokee_buffer_add (conn->local_directory, conn->request->buf, conn->request->len); /* (1) */
 	n->fd = open (conn->local_directory->buf, O_RDONLY);
-	cherokee_buffer_drop_endding (conn->local_directory, conn->request->len);
 
 	/* Cannot open the file
 	 */
-	ret = fstat (n->fd, &n->info);
+	if (n->fd != -1) {
+		ret = fstat (n->fd, &n->info);
+	} else {
+		ret = stat (conn->local_directory->buf, &n->info);
+	}
+
+	cherokee_buffer_drop_endding (conn->local_directory, conn->request->len); /* (1') */
+
 
 	if (n->fd == -1) {
 		if ((ret == 0) && S_ISREG(n->info.st_mode)) {
-			CONN(HANDLER(n)->connection)->error_code = http_access_denied;
+			conn->error_code = http_access_denied;
 		} else {
-			CONN(HANDLER(n)->connection)->error_code = http_not_found;
+			conn->error_code = http_not_found;
 		}
 		return ret_error;
 	}
@@ -89,23 +102,37 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 	/* Is dir?
 	 */
 	if (S_ISDIR(n->info.st_mode)) {
-		CONN(HANDLER(n)->connection)->error_code = http_access_denied;
+		conn->error_code = http_access_denied;
 		return ret_error;		
 	}
 
 
 	/* Range
 	 */
-	if (HANDLER_CONN(n)->range_start != 0) {
-		n->offset = HANDLER_CONN(n)->range_start;
+	if (conn->range_start != 0) {
+		n->offset = conn->range_start;
 		lseek (n->fd, n->offset, SEEK_SET);
-		HANDLER_CONN(n)->error_code = http_partial_content;
+		conn->error_code = http_partial_content;
 	}
 
-	if (HANDLER_CONN(n)->range_end == 0) {
-		HANDLER_CONN(n)->range_end = n->info.st_size;
+	if (conn->range_end == 0) {
+		conn->range_end = n->info.st_size;
 	}
 
+
+	/* Maybe use sendfile
+	 */
+#if HAVE_SENDFILE
+	n->using_sendfile = ((n->info.st_size >= 32768) && 
+			     (n->info.st_size < 2147483647) &&
+			     conn->encoder == NULL);
+
+	if (n->using_sendfile) {
+		cherokee_connection_set_cork(conn, 1);
+	}
+#endif
+
+
 	/* Look for the mime type
 	 */
 	ext = rindex (conn->request->buf, '.');
@@ -140,6 +167,39 @@ cherokee_handler_file_step (cherokee_handler_file_t *fhdl,
 {
 	int total;
 
+#if HAVE_SENDFILE
+	if (fhdl->using_sendfile) {
+		ssize_t sent;
+		cherokee_connection_t *conn = CONN(HANDLER(fhdl)->connection);
+		
+		sent = sendfile (SOCKET_FD(conn->socket),         /* int    out_fd */
+				 fhdl->fd,                        /* int     in_fd */
+				 &fhdl->offset,                   /* off_t *offset */
+				 conn->range_end - fhdl->offset); /* size_t  count */
+
+		/* cherokee_handler_file_init() activated the TCP_CORK flags.
+		 * After it, the header was sent.  And now, the first
+		 * chunk of the file with sendfile().  It's time to turn
+		 * off again the TCP_CORK flag
+		 */
+		if (conn->tcp_cork) {
+			cherokee_connection_set_cork(conn, 0);
+		}
+
+
+		if (sent < 0) {
+			return ret_error;
+		}
+
+		if (fhdl->offset >= conn->range_end) {
+			return ret_eof;
+		}
+
+		return ret_ok_and_sent;
+	}
+#endif
+
+
 	total = read (fhdl->fd, buffer->buf, buffer->size);	
 	switch (total) {
 	case 0:
