@@ -29,15 +29,44 @@
 
 #include "plugin_table.h"
 #include "handler.h"
+#include "static_handlers.h"
+#include "library.h"
+
+
+
+static ret_t
+load_static_table (cherokee_plugin_table_t *pt)
+{
+	int i;
+	
+	for (i=0; static_handlers[i].name; i++) {
+		ret_t ret;
+		ret = cherokee_table_add (&pt->handlers, static_handlers[i].name, static_handlers[i].func); 
+		if (ret != ret_ok) 
+			return ret;
+	}
+
+	return ret_ok;
+}
+
+
 
 ret_t
 cherokee_plugin_table_new  (cherokee_plugin_table_t **pt)
 {
-	cherokee_plugin_table_t *n = (cherokee_plugin_table_t *)malloc(sizeof(cherokee_plugin_table_t));
+	CHEROKEE_NEW_STRUCT(n, plugin_table);
 
+	/* Init tables 
+	 */
 	cherokee_table_init (&n->table);
 	cherokee_table_init (&n->handlers);
 	
+	/* Add the static modules
+	 */
+	load_static_table (n);
+
+	/* Return the object
+	 */
 	*pt = n;
 	
 	return ret_ok;
@@ -56,96 +85,71 @@ cherokee_plugin_table_free (cherokee_plugin_table_t *pt)
 }
 
 
-void *
-cherokee_plugin_table_get_from_url (cherokee_plugin_table_t *pt, char *dir)
+ret_t 
+cherokee_plugin_table_get (cherokee_plugin_table_t *pt, char *requested_url, cherokee_plugin_table_entry_t **plugin_entry, cherokee_buffer_t *web_directory)
 {
-	void *ret = NULL;
-	int   length;
 	char *slash;
-	char *url;
-	
-	url = strdup(dir);
-	
+	cherokee_plugin_table_entry_t *entry = NULL;
+
+	cherokee_buffer_add (web_directory, requested_url, strlen(requested_url));
+
 	do {
-		ret = cherokee_table_get (&pt->table, url);
+		entry = cherokee_table_get (&pt->table, web_directory->buf);
 		
 		/* Found
 		 */
-		if (ret != NULL)
+		if (entry != NULL)
 			goto go_out;
-		
-		length = strlen(url);
-		if (length == 0)
+ 
+		if (web_directory->len <= 0)
 			goto go_out;
-		
-		/* Modify url for next loop
+
+
+		/* Modify url for next loop:
+		 * Find the last / and finish the string there.
 		 */
-		if (url[length-1] == '/') {
-			url[length-1] = '\0';
+		if (web_directory->buf[web_directory->len-1] == '/') {
+			web_directory->buf[web_directory->len-1] = '\0';
+			web_directory->len--;
 		} else {
-			slash = rindex (url, '/');
+			slash = rindex (web_directory->buf, '/');
+			if (slash == NULL) goto go_out;
+
 			slash[1] = '\0';
+			web_directory->len -= ((web_directory->buf + web_directory->len) - slash -1);
 		}
-	} while (ret == NULL);
+
+	} while (entry == NULL);
 
 
-go_out:
-	free (url);
-	return ret;
+go_out:	
+	/* Set the in-out values
+	 */
+	*plugin_entry = entry;
+
+	return (entry == NULL) ? ret_error : ret_ok;
 }
 
 
 static inline func_new_t
 load_plugin_library (char *plugin_name)
 {
-	void       *lib;
-	func_new_t  func_new;
-	void      (*func_init) (void);
-	int         dl_flags;
+	cherokee_library_t *clib;
+	func_new_t ret;
+	char sym[200];
 	
-	/* Load the library
-	 */	
-#ifdef HAVE_RTLDNOW
-	dl_flags = RTLD_NOW;
-#else
-	dl_flags = RTLD_LAZY;
-#endif
+	cherokee_library_new (&clib, plugin_name);
 
-#ifdef HAVE_RTLDGLOBAL
-	dl_flags |= RTLD_GLOBAL;
-#endif
+ 	snprintf (sym, 200, "cherokee_handler_%s_new", plugin_name);
+	cherokee_library_get (clib, sym, (void **)&ret);
 
-	snprintf (gbl_buffer, gbl_buffer_size, "%s/libcherokee_%s." SO_SUFFIX, CHEROKEE_PLUGINDIR, plugin_name);
-	lib = dlopen (gbl_buffer, dl_flags);
-	if (lib == NULL) {
-		fprintf (stderr, "Can't read the plug-in (%s): %s\n", gbl_buffer, dlerror());
-		return NULL;
-	}
+	cherokee_library_free (clib);
 	
-
-	/* Maybe execute the library init function
-	 */
-	snprintf (gbl_buffer, gbl_buffer_size, "%s_init", plugin_name);
-	func_init = dlsym(lib, gbl_buffer);
-	if (func_init != NULL) {
-		func_init();
-	}
-
-
-	/* Get the constructor
-	 */
-	snprintf (gbl_buffer, gbl_buffer_size, "cherokee_handler_%s_new", plugin_name);
-	func_new = (func_new_t) dlsym(lib, gbl_buffer);
-	if (func_new == NULL) {
-		fprintf (stderr, "Can't load plug-in (%s): %s\n", gbl_buffer, dlerror());
-		return NULL;
-	}
-	
-	return func_new;
+	return ret;
 }
 
 
-static inline func_new_t
+static inline handler_func_new_t
 get_loaded_function (char *plugin_name)
 {
 	void (*func_init) (void);
@@ -167,46 +171,53 @@ get_loaded_function (char *plugin_name)
 
 	
 	snprintf (gbl_buffer, gbl_buffer_size, "cherokee_handler_%s_new", plugin_name);
-	return (func_new_t) dlsym(NULL, gbl_buffer);	
+	return (handler_func_new_t) dlsym(NULL, gbl_buffer);	
 }
 
 
 ret_t 
-cherokee_plugin_table_set (cherokee_plugin_table_t *pt, char *dir, char *handler)
+cherokee_plugin_table_fill_entry (cherokee_plugin_table_t *pt, char *handler_name, cherokee_plugin_table_entry_t *plugin_entry)
 {
-	ret_t      ret;
-	func_new_t func;
-
+	ret_t              ret;
+	handler_func_new_t func;
 
 	/* Get the cached "handler"_new function
 	 */
-	func = (func_new_t) cherokee_table_get (TABLE(&pt->handlers), handler);
+	func = (handler_func_new_t) cherokee_table_get (TABLE(&pt->handlers), handler_name);
 	if (func == NULL) {
 
 		/* Maybe the plugin is compiled statically
 		 */
-		func = (func_new_t) get_loaded_function(handler);
+		func = (handler_func_new_t) get_loaded_function (handler_name);
 
 		if (func == NULL) {
-			func = load_plugin_library (handler);
+			func = load_plugin_library (handler_name);
 			if (func == NULL) {
 				return ret_error;
 			}
 		}
 		
+		
 		/* Add to table
 		 */
-		ret = cherokee_table_add (&pt->handlers, handler, (void*)func); 
+		ret = cherokee_table_add (&pt->handlers, handler_name, (void*)func); 
 		if (ret != ret_ok) {
 			return ret_error;
 		}
 	}
 	
-	
-	/* Add "handler" <-> handler_new func
-	 */
-	cherokee_table_add (TABLE(&pt->table), dir, (void*)func);
-	
+	plugin_entry->handler_new_func = func;
+
 	return ret_ok;
 }
 
+
+ret_t inline
+cherokee_plugin_table_add (cherokee_plugin_table_t *pt, char *dir, cherokee_plugin_table_entry_t *plugin_entry)
+{
+	/* Add to "dir <-> plugin_entry" table
+	 */
+	return cherokee_table_add (TABLE(&pt->table), dir, (void*)plugin_entry);
+}
+
+
