@@ -26,19 +26,27 @@
 #ifndef __CHEROKEE_SERVER_H__
 #define __CHEROKEE_SERVER_H__
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else 
 #include <time.h>
+#endif
+
+#include <unistd.h>
+#include <sys/types.h>
 
 #include "common.h"
 #include "list.h"
 #include "fdpoll.h"
 #include "plugin_table.h"
 #include "virtual_server.h"
+#include "encoder_table.h"
+#include "logger_table.h"
+#include "thread.h"
 
 
 typedef struct {
 	time_t bogo_now;
-
-	cherokee_fdpoll_t         *fdpoll;
 	
 	/* Virtual servers
 	 */
@@ -47,18 +55,30 @@ typedef struct {
 	
 	/* lists
 	 */
-	list_t active_list;
-	list_t reuse_list;
+	list_t             thread_list;
+	cherokee_thread_t *main_thread;
+
+	/* Encoders 
+	 */
+	cherokee_encoder_table_t *encoders;
+
+	/* Logging
+	 */
+	int    log_flush_elapse;
+	time_t log_flush_next;
+	cherokee_logger_table_t  *loggers;
 
 	/* Configuration
 	 */
 	int   socket;
-	char *listen_to;
-
+	int   ipv6;
+	char *listen_to;	
 	int   fdwatch_msecs;
 
+	int   ncpus;
+	int   thread_num;
+
 	int   port;
-	int   log;
 	int   timeout;
 	int   keepalive;
 	int   hideversion;
