@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,14 +47,14 @@ static inline void
 del_item (void *avl_item, void *avl_param)
 {
 	free (((item_t *)avl_item)->key);
+	free ((item_t *)avl_item);
 }
 
 
 ret_t
 cherokee_table_new (cherokee_table_t **tab)
 {
-	cherokee_table_t *n = (cherokee_table_t *) malloc (sizeof(cherokee_table_t));
-	return_if_fail (n != NULL, ret_nomem);
+	CHEROKEE_NEW_STRUCT(n, table);
 	
 	cherokee_table_init (n);
 	
@@ -168,3 +168,29 @@ cherokee_table_del (cherokee_table_t  *tab, char *key)
 	fprintf (stderr, "cherokee_table_del: TODO\n");
 	return NULL;
 }
+
+
+ret_t 
+cherokee_table_foreach (cherokee_table_t *tab, cherokee_table_foreach_func_t func)
+{
+	struct avl_traverser  trav;
+	item_t               *item;
+
+	if (tab->tree == NULL) {
+		return ret_ok;
+	}
+
+	avl_t_init (&trav, tab->tree);
+
+	item = (item_t *) avl_t_first (&trav, tab->tree);
+	if (item != NULL) {
+		func (item->key, item->value);
+	}
+
+	while ((item = avl_t_next(&trav)) != NULL) {
+		func (item->key, item->value);
+	}
+
+	return ret_ok;
+}
+
