@@ -23,8 +23,8 @@
  * USA
  */
 
-#ifndef __CHEROKEE2_TABLE_H__
-#define __CHEROKEE2_TABLE_H__
+#ifndef __CHEROKEE_TABLE_H__
+#define __CHEROKEE_TABLE_H__
 
 #include "avl.h"
 #include "common.h"
@@ -33,23 +33,25 @@ typedef struct {
 	struct avl_table *tree;
 } cherokee_table_t;
 
-
-typedef void (* cherokee_table_free_item_t) (void *item);
-
+typedef void (* cherokee_table_free_item_t)    (void *item);
+typedef void (* cherokee_table_foreach_func_t) (const char *key, void *value);
 
 #define TABLE(x) ((cherokee_table_t *)(x))
 
-ret_t cherokee_table_new  (cherokee_table_t **tab);
-ret_t cherokee_table_init (cherokee_table_t  *tab);
 
-ret_t cherokee_table_free (cherokee_table_t  *tab);
-ret_t cherokee_table_free2(cherokee_table_t  *tab, cherokee_table_free_item_t func);
+ret_t cherokee_table_new    (cherokee_table_t **tab);
+ret_t cherokee_table_init   (cherokee_table_t  *tab);
 
-ret_t cherokee_table_clean (cherokee_table_t  *tab);
-ret_t cherokee_table_clean2(cherokee_table_t  *tab, cherokee_table_free_item_t func);
+ret_t cherokee_table_free   (cherokee_table_t  *tab);
+ret_t cherokee_table_free2  (cherokee_table_t  *tab, cherokee_table_free_item_t func);
 
-ret_t cherokee_table_add   (cherokee_table_t  *tab, char *key, void *value);
-void* cherokee_table_get   (cherokee_table_t  *tab, char *key);
-void* cherokee_table_del   (cherokee_table_t  *tab, char *key);
+ret_t cherokee_table_clean  (cherokee_table_t  *tab);
+ret_t cherokee_table_clean2 (cherokee_table_t  *tab, cherokee_table_free_item_t func);
 
-#endif /* __CHEROKEE2_TABLE_H__ */
+ret_t cherokee_table_add    (cherokee_table_t  *tab, char *key, void *value);
+void* cherokee_table_get    (cherokee_table_t  *tab, char *key);
+void* cherokee_table_del    (cherokee_table_t  *tab, char *key);
+
+ret_t cherokee_table_foreach (cherokee_table_t *tab, cherokee_table_foreach_func_t func);
+
+#endif /* __CHEROKEE_TABLE_H__ */
